@@ -36,6 +36,7 @@
 #include "linglong/utils/error/error.h"
 #include "linglong/utils/finally/finally.h"
 #include "linglong/utils/gettext.h"
+#include "linglong/utils/log/log.h"
 #include "linglong/utils/serialize/json.h"
 #include "linglong/utils/xdg/directory.h"
 #include "ocppi/runtime/ExecOption.hpp"
@@ -553,7 +554,7 @@ void Cli::printProgress() noexcept
             return;
         }
 
-        if (options.verbose) {
+        if (this->globalOptions.verbose) {
             this->printer.printTaskState(this->lastPercentage,
                                          this->lastMessage,
                                          this->lastState,
@@ -603,7 +604,7 @@ Cli::Cli(Printer &printer,
     }
 }
 
-int Cli::run([[maybe_unused]] CLI::App *subcommand)
+int Cli::run(const RunOptions &options)
 {
     LINGLONG_TRACE("command run");
 
@@ -659,9 +660,23 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
     }
 
     runtime::RunContext runContext(this->repository);
-    auto res = runContext.resolve(*curAppRef);
+    linglong::runtime::ResolveOptions opts;
+    opts.baseRef = options.base;
+    opts.runtimeRef = options.runtime;
+
+    LogD("start resolve run context with base {} and runtime {}",
+         opts.baseRef.value_or("null"),
+         opts.runtimeRef.value_or("null"));
+
+    auto res = runContext.resolve(*curAppRef, opts);
     if (!res) {
         this->printer.printErr(res.error());
+        return -1;
+    }
+
+    LogD("resolved run context with base {}", runContext.getBaseLayerPath()->string());
+    if (runContext.hasRuntime()) {
+        LogD("resolved run context with runtime {}", runContext.getRuntimeLayerPath()->string());
     }
 
     const auto &appLayerItem = runContext.getCachedAppItem();
@@ -679,7 +694,7 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
     if (options.commands.empty()) {
         commands = info.command.value_or(std::vector<std::string>{ "bash" });
     }
-    commands = filePathMapping(commands);
+    commands = filePathMapping(commands, options);
 
     // this lambda will dump reference of containerID, app, base and runtime to
     // /run/linglong/getuid()/getpid() to store these needed infomation
@@ -758,7 +773,7 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
       .bindCgroup()
       .bindRun()
       .bindUserGroup()
-      .bindMedia()
+      .bindRemovableStorageMounts()
       .bindHostRoot()
       .bindHostStatics()
       .bindHome(homeEnv)
@@ -822,7 +837,7 @@ int Cli::run([[maybe_unused]] CLI::App *subcommand)
     return 0;
 }
 
-int Cli::exec([[maybe_unused]] CLI::App *subcommand)
+int Cli::enter(const EnterOptions &options)
 {
     LINGLONG_TRACE("ll-cli exec");
     auto containers = getCurrentContainers();
@@ -941,7 +956,7 @@ Cli::getCurrentContainers() const noexcept
     return myContainers;
 }
 
-int Cli::ps([[maybe_unused]] CLI::App *subcommand)
+int Cli::ps()
 {
     auto myContainers = getCurrentContainers();
     if (!myContainers) {
@@ -961,7 +976,7 @@ int Cli::ps([[maybe_unused]] CLI::App *subcommand)
     return 0;
 }
 
-int Cli::kill([[maybe_unused]] CLI::App *subcommand)
+int Cli::kill(const KillOptions &options)
 {
     LINGLONG_TRACE("command kill");
 
@@ -1012,7 +1027,9 @@ void Cli::cancelCurrentTask()
     }
 }
 
-int Cli::installFromFile(const QFileInfo &fileInfo, const api::types::v1::CommonOptions &options)
+int Cli::installFromFile(const QFileInfo &fileInfo,
+                         const api::types::v1::CommonOptions &commonOptions,
+                         const std::string &appid)
 {
     auto filePath = fileInfo.absoluteFilePath();
     LINGLONG_TRACE(QString{ "install from file %1" }.arg(filePath));
@@ -1024,7 +1041,7 @@ int Cli::installFromFile(const QFileInfo &fileInfo, const api::types::v1::Common
         // 所以将layer或uab文件的相对路径转为绝对路径，再传给pkexec
         auto path = fileInfo.absoluteFilePath();
         for (auto i = 0; i < args.length(); i++) {
-            if (args[i] == QString::fromStdString(this->options.appid)) {
+            if (args[i] == QString::fromStdString(appid)) {
                 args[i] = path.toLocal8Bit().constData();
             }
         }
@@ -1058,9 +1075,10 @@ int Cli::installFromFile(const QFileInfo &fileInfo, const api::types::v1::Common
 
     QDBusUnixFileDescriptor dbusFileDescriptor(file.handle());
 
-    auto pendingReply = this->pkgMan.InstallFromFile(dbusFileDescriptor,
-                                                     fileInfo.suffix(),
-                                                     utils::serialize::toQVariantMap(options));
+    auto pendingReply =
+      this->pkgMan.InstallFromFile(dbusFileDescriptor,
+                                   fileInfo.suffix(),
+                                   utils::serialize::toQVariantMap(commonOptions));
     pendingReply.waitForFinished();
     if (pendingReply.isError()) {
         if (pendingReply.error().type() == QDBusError::AccessDenied) {
@@ -1118,7 +1136,7 @@ int Cli::installFromFile(const QFileInfo &fileInfo, const api::types::v1::Common
     return this->lastState == linglong::api::types::v1::State::Succeed ? 0 : -1;
 }
 
-int Cli::install([[maybe_unused]] CLI::App *subcommand)
+int Cli::install(const InstallOptions &options)
 {
     LINGLONG_TRACE("command install");
 
@@ -1134,7 +1152,7 @@ int Cli::install([[maybe_unused]] CLI::App *subcommand)
 
     // 如果检测是文件，则直接安装
     if (info.exists() && info.isFile()) {
-        return installFromFile(QFileInfo{ info.absoluteFilePath() }, params.options);
+        return installFromFile(QFileInfo{ info.absoluteFilePath() }, params.options, options.appid);
     }
 
     QDBusReply<QString> authReply = this->authorization();
@@ -1245,7 +1263,7 @@ int Cli::install([[maybe_unused]] CLI::App *subcommand)
             return -1;
         }
 
-        if (options.verbose) {
+        if (this->globalOptions.verbose) {
             this->printer.printReply({ .code = result->code, .message = result->message });
         }
 
@@ -1278,7 +1296,7 @@ int Cli::install([[maybe_unused]] CLI::App *subcommand)
     return this->lastState == linglong::api::types::v1::State::Succeed ? 0 : -1;
 }
 
-int Cli::upgrade([[maybe_unused]] CLI::App *subcommand)
+int Cli::upgrade(const UpgradeOptions &options)
 {
     LINGLONG_TRACE("command upgrade");
 
@@ -1417,7 +1435,7 @@ int Cli::upgrade([[maybe_unused]] CLI::App *subcommand)
     return 0;
 }
 
-int Cli::search([[maybe_unused]] CLI::App *subcommand)
+int Cli::search(const SearchOptions &options)
 {
     LINGLONG_TRACE("command search");
 
@@ -1432,7 +1450,7 @@ int Cli::search([[maybe_unused]] CLI::App *subcommand)
         // 检查repo是否存在
         auto it = std::find_if(repoConfig.repos.begin(),
                                repoConfig.repos.end(),
-                               [this](const api::types::v1::Repo &repo) {
+                               [this, &options](const api::types::v1::Repo &repo) {
                                    return repo.alias.value_or(repo.name) == options.repo.value();
                                });
         if (it == repoConfig.repos.end()) {
@@ -1482,7 +1500,7 @@ int Cli::search([[maybe_unused]] CLI::App *subcommand)
     connect(
       &this->pkgMan,
       &api::dbus::v1::PackageManager::SearchFinished,
-      [&pendingJobID, this, &loop](const QString &jobID, const QVariantMap &data) {
+      [&pendingJobID, this, &loop, &options](const QString &jobID, const QVariantMap &data) {
           LINGLONG_TRACE("process search result");
           // Note: once an error occurs, remember to return after exiting the loop.
           if (!pendingJobID || *pendingJobID != jobID) {
@@ -1511,7 +1529,7 @@ int Cli::search([[maybe_unused]] CLI::App *subcommand)
                                                "\n2. Verify network proxy settings if used"));
               }
 
-              if (options.verbose) {
+              if (this->globalOptions.verbose) {
                   this->printer.printErr(
                     LINGLONG_ERRV("\n" + QString::fromStdString(result->message), result->code));
               }
@@ -1557,7 +1575,7 @@ int Cli::search([[maybe_unused]] CLI::App *subcommand)
     return loop.exec();
 }
 
-int Cli::prune([[maybe_unused]] CLI::App *subcommand)
+int Cli::prune()
 {
     LINGLONG_TRACE("command prune");
 
@@ -1625,7 +1643,7 @@ int Cli::prune([[maybe_unused]] CLI::App *subcommand)
     return loop.exec();
 }
 
-int Cli::uninstall([[maybe_unused]] CLI::App *subcommand)
+int Cli::uninstall(const UninstallOptions &options)
 {
     LINGLONG_TRACE("command uninstall");
 
@@ -1644,44 +1662,7 @@ int Cli::uninstall([[maybe_unused]] CLI::App *subcommand)
         return -1;
     }
 
-    auto ref = this->repository.clearReference(*fuzzyRef,
-                                               {
-                                                 .forceRemote = false,
-                                                 .fallbackToRemote = false,
-                                               });
-    if (!ref) {
-        const auto errCode = static_cast<utils::error::ErrorCode>(ref.error().code());
-        if (errCode == utils::error::ErrorCode::AppNotFoundFromLocal) {
-            this->printer.printMessage(_("Application is not installed."));
-
-            if (options.verbose) {
-                this->printer.printErr(ref.error());
-            }
-            return -1;
-        }
-        this->printer.printErr(ref.error());
-        return -1;
-    }
-
-    std::string module = "binary";
     auto params = api::types::v1::PackageManager1UninstallParameters{};
-    if (!options.module.empty()) {
-        module = options.module;
-        params.package.packageManager1PackageModule = module;
-    }
-
-    auto layerItem = this->repository.getLayerItem(*ref, module);
-    if (!layerItem) {
-        this->printer.printErr(layerItem.error());
-        return -1;
-    }
-
-    if (layerItem->info.kind != "app") {
-        this->printer.printErr(
-          LINGLONG_ERRV("This layer is not an application, please use 'll-cli prune'.", -1));
-        return -1;
-    }
-
     params.package.id = fuzzyRef->id.toStdString();
     if (fuzzyRef->channel) {
         params.package.channel = fuzzyRef->channel->toStdString();
@@ -1689,6 +1670,9 @@ int Cli::uninstall([[maybe_unused]] CLI::App *subcommand)
     if (fuzzyRef->version) {
         params.package.version = fuzzyRef->version->toStdString();
     }
+    if (!options.module.empty()) {
+        params.package.packageManager1PackageModule = options.module;
+    }
 
     auto pendingReply = this->pkgMan.Uninstall(utils::serialize::toQVariantMap(params));
     pendingReply.waitForFinished();
@@ -1727,6 +1711,21 @@ int Cli::uninstall([[maybe_unused]] CLI::App *subcommand)
         case utils::error::ErrorCode::AppUninstallNotFoundFromLocal:
             this->printer.printMessage(_("Application is not installed."));
             break;
+        case utils::error::ErrorCode::AppUninstallMultipleVersions:
+            this->printer.printMessage(
+              QString{ _("Multiple versions of the package are installed. Please specify a single "
+                         "version to uninstall:\n%1") }
+                .arg(result->message.c_str()));
+            break;
+        case utils::error::ErrorCode::AppUninstallAppIsRunning:
+            this->printer.printMessage(
+              _("The application is currently running and cannot be "
+                "uninstalled. Please turn off the application and try again."));
+            break;
+        case utils::error::ErrorCode::AppUninstallBaseOrRuntime:
+            this->printer.printMessage(
+              _("Base or runtime cannot be uninstalled, please use 'll-cli prune'."));
+            break;
         case utils::error::ErrorCode::AppUninstallFailed:
         case utils::error::ErrorCode::Unknown:
             this->printer.printMessage(_("Uninstall failed"));
@@ -1736,7 +1735,7 @@ int Cli::uninstall([[maybe_unused]] CLI::App *subcommand)
             return -1;
         }
 
-        if (options.verbose) {
+        if (this->globalOptions.verbose) {
             this->printer.printErr(err);
         }
 
@@ -1770,39 +1769,44 @@ int Cli::uninstall([[maybe_unused]] CLI::App *subcommand)
     return this->lastState == linglong::api::types::v1::State::Succeed ? 0 : -1;
 }
 
-int Cli::list([[maybe_unused]] CLI::App *subcommand)
-
+int Cli::list(const ListOptions &options)
 {
-    if (!options.showUpgradeList) {
-        auto items = this->repository.listLayerItem();
-        if (!items) {
-            this->printer.printErr(items.error());
+    if (options.showUpgradeList) {
+        auto upgradeList = this->listUpgradable();
+        if (!upgradeList) {
+            this->printer.printErr(upgradeList.error());
             return -1;
         }
-        std::vector<api::types::v1::PackageInfoDisplay> list;
-        for (const auto &item : *items) {
-            nlohmann::json json = item.info;
-            auto m = json.get<api::types::v1::PackageInfoDisplay>();
-            auto t = this->repository.getLayerCreateTime(item);
-            if (t.has_value()) {
-                m.installTime = *t;
-            }
-            list.push_back(std::move(m));
-        }
-        if (!options.type.empty()) {
-            filterPackageInfosByType(list, options.type);
-        }
-        this->printer.printPackages(list);
+        // 按id排序
+        std::sort(upgradeList->begin(), upgradeList->end(), [](const auto &lhs, const auto &rhs) {
+            return lhs.id < rhs.id;
+        });
+        this->printer.printUpgradeList(*upgradeList);
         return 0;
     }
-
-    auto upgradeList = this->listUpgradable();
-    if (!upgradeList) {
-        this->printer.printErr(upgradeList.error());
+    auto items = this->repository.listLayerItem();
+    if (!items) {
+        this->printer.printErr(items.error());
         return -1;
     }
-
-    this->printer.printUpgradeList(*upgradeList);
+    std::vector<api::types::v1::PackageInfoDisplay> list;
+    for (const auto &item : *items) {
+        nlohmann::json json = item.info;
+        auto m = json.get<api::types::v1::PackageInfoDisplay>();
+        auto t = this->repository.getLayerCreateTime(item);
+        if (t.has_value()) {
+            m.installTime = *t;
+        }
+        list.push_back(std::move(m));
+    }
+    if (!options.type.empty()) {
+        filterPackageInfosByType(list, options.type);
+    }
+    // 按id排序
+    std::sort(list.begin(), list.end(), [](const auto &lhs, const auto &rhs) {
+        return lhs.id < rhs.id;
+    });
+    this->printer.printPackages(list);
     return 0;
 }
 
@@ -1881,7 +1885,7 @@ Cli::listUpgradable(const std::string &type)
     return upgradeList;
 }
 
-int Cli::repo(CLI::App *app)
+int Cli::repo(CLI::App *app, const RepoOptions &options)
 {
     LINGLONG_TRACE("command repo");
 
@@ -1922,7 +1926,7 @@ int Cli::repo(CLI::App *app)
         return EINVAL;
     }
 
-    std::string url = options.repoOptions.repoUrl;
+    std::string url = options.repoUrl;
 
     if (argsParseFunc("add") || argsParseFunc("update")) {
         if (url.rfind("http", 0) != 0) {
@@ -1936,9 +1940,9 @@ int Cli::repo(CLI::App *app)
         }
     }
 
-    std::string name = options.repoOptions.repoName;
+    std::string name = options.repoName;
     // if alias is not set, use name as alias
-    std::string alias = options.repoOptions.repoAlias.value_or(name);
+    std::string alias = options.repoAlias.value_or(name);
     auto &cfgRef = *cfg;
 
     if (argsParseFunc("add")) {
@@ -1957,7 +1961,7 @@ int Cli::repo(CLI::App *app)
             return -1;
         }
         cfgRef.repos.push_back(api::types::v1::Repo{
-          .alias = options.repoOptions.repoAlias,
+          .alias = options.repoAlias,
           .name = name,
           .priority = 0,
           .url = url,
@@ -2027,7 +2031,7 @@ int Cli::repo(CLI::App *app)
     }
 
     if (argsParseFunc("set-priority")) {
-        existingRepo->priority = options.repoOptions.repoPriority;
+        existingRepo->priority = options.repoPriority;
         return this->setRepoConfig(utils::serialize::toQVariantMap(cfgRef));
     }
 
@@ -2063,7 +2067,7 @@ int Cli::setRepoConfig(const QVariantMap &config)
     return 0;
 }
 
-int Cli::info([[maybe_unused]] CLI::App *subcommand)
+int Cli::info(const InfoOptions &options)
 {
     LINGLONG_TRACE("command info");
 
@@ -2129,7 +2133,7 @@ int Cli::info([[maybe_unused]] CLI::App *subcommand)
     return 0;
 }
 
-int Cli::content([[maybe_unused]] CLI::App *subcommand)
+int Cli::content(const ContentOptions &options)
 {
     LINGLONG_TRACE("command content");
 
@@ -2245,8 +2249,8 @@ int Cli::content([[maybe_unused]] CLI::App *subcommand)
     return std::string{ url };
 }
 
-std::vector<std::string>
-Cli::filePathMapping(const std::vector<std::string> &command) const noexcept
+std::vector<std::string> Cli::filePathMapping(const std::vector<std::string> &command,
+                                              const RunOptions &options) const noexcept
 {
     // FIXME: couldn't handel command like 'll-cli run org.xxx.yyy --file f1 f2 f3 org.xxx.yyy %%F'
     // can't distinguish the boundary of command , need validate the command arguments in the future
@@ -2757,7 +2761,7 @@ void Cli::updateAM() noexcept
     }
 }
 
-int Cli::inspect([[maybe_unused]] CLI::App *subcommand)
+int Cli::inspect(const InspectOptions &options)
 {
     auto myContainersRet = getCurrentContainers();
     if (!myContainersRet) {
@@ -2768,10 +2772,10 @@ int Cli::inspect([[maybe_unused]] CLI::App *subcommand)
 
     api::types::v1::InspectResult result;
 
-    if (this->options.pid) {
-        qDebug() << "inspect by pid:" << this->options.pid.value();
+    if (options.pid) {
+        qDebug() << "inspect by pid:" << options.pid.value();
         for (const auto &container : myContainers) {
-            auto ret = isChildProcess(container.pid, this->options.pid.value());
+            auto ret = isChildProcess(container.pid, options.pid.value());
             if (!ret) {
                 this->printer.printErr(ret.error());
                 return -1;
@@ -2788,7 +2792,7 @@ int Cli::inspect([[maybe_unused]] CLI::App *subcommand)
     return 0;
 }
 
-int Cli::dir([[maybe_unused]] CLI::App *subcommand)
+int Cli::dir(const DirOptions &options)
 {
     LINGLONG_TRACE("command dir");
 
